package ru.ulstu.fc.rule.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import ru.ulstu.fc.rule.model.AddRuleForm;
import ru.ulstu.fc.rule.model.AddTermForm;
import ru.ulstu.fc.rule.model.AddVariableForm;
import ru.ulstu.fc.rule.service.RuleParseService;
import ru.ulstu.fc.rule.service.RuleService;
import ru.ulstu.fc.rule.service.TermsService;
import ru.ulstu.fc.rule.service.VariableService;

import java.util.List;

@Controller
public class RuleController {
    private final RuleParseService ruleParseService;
    private final RuleService ruleService;
    private final TermsService termService;
    private final VariableService variableService;

    public RuleController(RuleParseService ruleParseService,
                          RuleService ruleService,
                          TermsService termService,
                          VariableService variableService) {
        this.ruleParseService = ruleParseService;
        this.ruleService = ruleService;
        this.termService = termService;
        this.variableService = variableService;
    }

    @GetMapping("listRules")
    public String listRules(Model model) {
        model.addAttribute("rules", ruleService.getRules());
        return "listRules";
    }

    @GetMapping("listTerms")
    public String listTerms(Model model) {
        model.addAttribute("terms", termService.getTerms());
        return "listTerms";
    }

    @GetMapping("listVars")
    public String listVariables(Model model) {
        model.addAttribute("vars", variableService.getVars());
        return "listVars";
    }

    @GetMapping("addRule")
    public String addRule(Model model) {
        model.addAttribute("addRuleForm", new AddRuleForm());
        return "addRule";
    }

    @PostMapping("addRule")
    public String parse(@ModelAttribute AddRuleForm addRuleForm, Model model) {
        try {
            System.out.println(ruleParseService.parseRules(List.of(addRuleForm.getRule())));
        } catch (Exception ex) {
            return "addRule";
        }
        model.addAttribute("addRuleForm", addRuleForm);
        return "listRules";
    }

    @GetMapping("addVariable")
    public String addVariable(Model model, @RequestParam(required = false) Integer id) {
        model.addAttribute("addVariableForm", variableService.getAddVariableFormOrDefault(id));
        return "addVariable";
    }

    @PostMapping("addVariable")
    public String addVariable(@ModelAttribute AddVariableForm addVariableForm, Model model) {
        model.addAttribute("addVariableForm", addVariableForm);
        variableService.save(addVariableForm);
        model.addAttribute("vars", variableService.getVars());
        return "listVars";
    }

    @GetMapping("addTerm")
    public String addTerm(Model model) {
        model.addAttribute("addTermForm", new AddTermForm());
        return "addTerm";
    }

    @PostMapping("addTerm")
    public String addTerm(@ModelAttribute AddTermForm addTermForm, Model model) {
        model.addAttribute("addTermForm", addTermForm);
        return "listTerms";
    }
}
